﻿using Newtonsoft.Json;
using Oxide.Core;
using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

#pragma warning disable CS0162

namespace Oxide.Plugins
{
    [Info("NoEscape", "Kaidoz | vk.com/kaidoz", "1.1.2")]
    [Description("Плагин запрещающий определенные действия по время рейда.")]
    public class NoEscape : HurtworldPlugin
    {

        public class OnRaid
        {
            public ulong steamid { get; set; }
            public string time { get; set; }
            public OnRaid(ulong steamid, string time)
            {
                this.steamid = steamid;
                this.time = time;
            }
        }

        List<OnRaid> onraids = new List<OnRaid>();

        EItemCode[] ForbiddenItems = new EItemCode[] {
            EItemCode.ConstructionHammer, EItemCode.OwnershipStake,
            EItemCode.StorageLocker, EItemCode.StorageChest, EItemCode.Workbench,
            EItemCode.OwrongSeeds, EItemCode.CottonSeed, EItemCode.IrradiatedOwrongSeeds, EItemCode.MedusaSeeds, EItemCode.PitcherPlantSeeds, EItemCode.SucculentSeeds,
            EItemCode.FirePit, EItemCode.Fireplace, EItemCode.Blastfurnace,
            EItemCode.AutomaticDrill, EItemCode.AutomaticDrillBlue, EItemCode.AutomaticDrillGreen, EItemCode.LandcrabMine, EItemCode.PoisonTrap,
            EItemCode.Fridge, EItemCode.BasicShack
        };

        Configuration _config;

        private void LoadDefaultMessages()
        {
            var messages = new Dictionary<string, string>
            {
                {"RaidBlock Started", "<color=red><b>[NoEscape]</b></color> Начался рейд! У вас блок {0} минут"},
                {"Message about Timer", "<color=red><b>[NoEscape]</b></color> У вас блок на {0} секунд."}
            };

            lang.RegisterMessages(messages, this);
        }

        private new void LoadConfig()
        {
            base.LoadConfig();
            try
            {
                _config = Config.ReadObject<Configuration>();
                if (_config == null) throw new Exception();
            }
            catch
            {
                Config.WriteObject(_config, false, $"{Interface.Oxide.ConfigDirectory}/{Name}.jsonError");
                PrintError("The configuration file contains an error and has been replaced with a default config.\n" +
                           "The error configuration file was saved in the .jsonError extension");
                LoadDefaultConfig();
            }
            SaveConfig();
        }

        protected override void LoadDefaultConfig()
        {
            PrintWarning("Creating new config file for NoEscape...");
            _config = new Configuration()
            {
                runforadmin = true,
                noblockmeteor = 10,
                exployactiveblock = false,
                logs = true,
                timec4 = 5,
                timedrill  = 7,
                radius = 150
            };
            SaveConfig();
        }

        protected override void SaveConfig() => Config.WriteObject(_config);

        public class Configuration
        {
            [JsonProperty("RaidBlock for Admins")]
            public bool runforadmin { get; set; }

            [JsonProperty("NoBlockMeteor(not enabled)")]
            public int noblockmeteor { get; set; }

            [JsonProperty("RaidBlock at Explosixe(not enabled)")]
            public bool exployactiveblock { get; set; }

            [JsonProperty("Logs")]
            public bool logs { get; set; }

            [JsonProperty("RaidBlock time of C4(not enabled)")]
            public int timec4 { get; set; }

            [JsonProperty("RaidBlock time of RaidDrill(not enabled)")]
            public int timedrill { get; set; }

            [JsonProperty("Radius of RaidBlock")]
            public int radius { get; set; }
        }


        void SaveData() => Interface.Oxide.DataFileSystem.WriteObject("NoEscape/players", onraids);

        void LoadData()
        {
            if (Interface.Oxide.DataFileSystem.ExistsDatafile("NoEscape/players"))
                onraids = Interface.Oxide.DataFileSystem.ReadObject<List<OnRaid>>("NoEscape/players");

            if (onraids.Count() == 0)
                return;
            for (int o = 0; o < onraids.Count(); o++)
            {
                try
                {
                    if (DateTime.Compare(DateTime.Parse(onraids[o].time), DateTime.Now) < 0)
                        onraids.Remove(onraids[o]);
                }
                catch { }
            }
            SaveData();

        }

        #region Hooks

        void Loaded()
        {
            LoadData();
            LoadConfig();
        }

        void OnPlayerInput(PlayerSession session)
        {
            if (session.WorldPlayerEntity!=null && israida(session))
                checkitem(session);
        }

        void OnEntitySpawned(NetworkViewData data)
        {
            if (_config.exployactiveblock) // -- Not Work
            {
                return;
                if (data.NetworkView.gameObject.name.Contains("ExplosionServer") && nometeor(data.NetworkView.gameObject, _config.noblockmeteor))
                {
                    ExplosionServer explosion = data.NetworkView.gameObject.GetComponent<ExplosionServer>();
                    var sessions = GameManager.Instance?.GetSessions()?.Values.Where(IsValidSession).ToList();
                    List<PlayerSession> playersNear =
                      sessions.Where(s =>
                              Vector3.Distance(s.WorldPlayerEntity.transform.position,
                                  data.NetworkView.gameObject.transform.position) <= _config.radius)
                          .ToList();
                    blockplayers(playersNear, _config.timec4);
                }
            }
            else
            {
                if (data.NetworkView.gameObject.name.Contains("C4"))
                {
                    var sessions = GameManager.Instance?.GetSessions()?.Values.Where(IsValidSession).ToList();
                    var playersNear =
                      sessions.Where(s =>
                              Vector3.Distance(s.WorldPlayerEntity.transform.position,
                                  data.NetworkView.gameObject.transform.position) <= _config.radius)
                          .ToList();
                    blockplayers(playersNear, _config.timec4);

                }
            }

            if ((data.NetworkView.gameObject.name.Contains("RaidDrill") && !data.NetworkView.gameObject.name.Contains("WorldItem")))
            {
                var sessions = GameManager.Instance?.GetSessions()?.Values.Where(IsValidSession).ToList();
                var playersNear =
                  sessions.Where(s =>
                          Vector3.Distance(s.WorldPlayerEntity.transform.position,
                              data.NetworkView.gameObject.transform.position) <= _config.radius)
                      .ToList();
                blockplayers(playersNear, _config.timedrill);

            }
        }

        #endregion

        #region methods

        void addraid(PlayerSession session, int time)
        {
            clearexist((ulong)session.SteamId, time);
            DateTime blocktime = DateTime.Now.AddMinutes(time);
            onraids.Add(new OnRaid((ulong)session.SteamId, blocktime.ToString()));
            reply(session, string.Format(Msg("RaidBlock Started", session.SteamId.ToString()),time));
            checkitem(session);
            SaveData();
        }

        void checkitem(PlayerSession session)
        {
            var ehs = session.WorldPlayerEntity.GetComponent<EquippedHandlerServer>();
            var item = ehs.GetEquippedItem();
            if (item == null)
                return;

            if (CheckItem(item.Item.ItemId))
            {
                A:
                if ((from x in onraids where x.steamid == (ulong)session.SteamId select x).Count() > 0)
                {
                    var dat = (from x in onraids where x.steamid == (ulong)session.SteamId select x).FirstOrDefault();
                    int result = DateTime.Compare(DateTime.Parse(dat.time), DateTime.Now);
                    Puts(result.ToString());
                    if (result > 0)
                    {
                        int countraid = Convert.ToInt32(DateTime.Parse(dat.time).Subtract(DateTime.Now).TotalSeconds);
                        reply(session, string.Format(Msg("Message about Timer", session.SteamId.ToString()),countraid));//$"<color=red><b>[NoEscape]</b></color> У вас блок на {countraid} секунд.");
                        checkinventory(session);
                    }
                    else
                    {
                        onraids.Remove(dat);
                        SaveData();
                        goto A;
                    }
                }
            }
        }

        void checkinventory(PlayerSession session)
        {
            var ehs = session.WorldPlayerEntity.GetComponent<EquippedHandlerServer>();
            var item = ehs.GetEquippedItem();

            if ((from x in ForbiddenItems where (int)x == item.Item.ItemId select x).Count() == 0)
                return;

            dropslot(session, ehs.GetEquippedSlotNumber());
        }

        void dropslot(PlayerSession session, int slot)
        {
            var inv = session.WorldPlayerEntity.GetComponent<PlayerInventory>();
            var item = inv.Items[slot];
            inv.Items[slot] = null;
            inv.Invalidate(false);
            createcache(session.WorldPlayerEntity.transform.position, item.Item.ItemId, item.StackSize);
        }


        /// <summary>
        /// Тестовая проверка. Возможно будет нужна,когда реализую блокировку при взрыве
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="radius"></param>
        /// <returns></returns>
        bool nometeor(GameObject obj, int radius)
        {
            var lst = Resources.FindObjectsOfTypeAll<GameObject>().ToList();

            List<GameObject> lstmeteors = (from x in lst where x.name.Contains("WorldItemMeteor") select x).ToList();
            if (lstmeteors.Count == 0)
                return true;

            foreach (GameObject _d in lstmeteors)
            {
                if (Vector3.Distance(_d.transform.position, obj.transform.position) <= radius)
                {
                    if (_d.GetComponent<MeteorController>().enabled) // For plugin EventTreasure
                    {
                        return false;
                    }
                }
            }
            return true;
        }

        bool israida(PlayerSession session)
        {
            if ((from x in onraids where x.steamid == (ulong)session.SteamId select x).Count() > 0)
                return true;
            return false;
        }


        void clearexist(ulong steamid, int time)
        {
            var d = (from x in onraids where x.steamid == steamid select x);
            while (d.Count() > 0)
            {
                int countraid = Convert.ToInt32(DateTime.Parse(d.FirstOrDefault().time).Subtract(DateTime.Now).Minutes);
                if (countraid > time)
                    break;

                onraids.Remove(d.FirstOrDefault());
                d = (from x in onraids where x.steamid == steamid select x);
            }
        }

        void blockplayers(List<PlayerSession> playersNear, int time)
        {
            foreach (PlayerSession session in playersNear)
            {
                if (session.IsAdmin)
                {
                    if (_config.runforadmin)
                        addraid(session, time);
                }
                else
                    addraid(session, time);
            }
            if (_config.logs)
                Logs(playersNear);
        }

        #endregion

        void reply(PlayerSession session, string msg) => hurt.SendChatMessage(session, null, msg);


        /// <summary>
        /// Проверка находится ли игрок в блокировке или нет
        /// </summary>
        /// <param name="session"></param>
        /// <returns></returns>
        bool israid(PlayerSession session)
        {
            A:
            if (israida(session))
            {
                var _fira = (from x in onraids where x.steamid == (ulong)session.SteamId select x);
                var dat = _fira.FirstOrDefault();
                int countraid = Convert.ToInt32(DateTime.Parse(dat.time).Subtract(DateTime.Now).TotalSeconds);
                if (countraid > 0)
                {
                    reply(session, string.Format(Msg("Message about Timer", session.SteamId.ToString()), countraid));
                    return true;
                }
                else
                {
                    onraids.Remove(dat);
                    SaveData();
                    goto A;
                }
            }

            return false;
        }

        #region Block Kits and TP
        object canExtTeleport(PlayerSession session)
        {
            if (israid(session))
            {
                return true;
            }
            return null;
        }


        /// <summary>
        /// Хук для заперта телепорта при рейдблоке
        /// </summary>
        /// <param name="session"></param>
        /// <returns></returns>
        object canRedeemKit(PlayerSession session)
        {
            if (israid(session))
            {
                return true;
            }
            return null;
        }
        #endregion

        #region Helpers

        string Msg(string msg, string SteamId = null) => lang.GetMessage(msg, this, SteamId);


        /// <summary>
        /// Проверка предмета на совпадение с ForbiddenItems
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        bool CheckItem(int id)
        {
            int count = (from x in ForbiddenItems where (int)x == id select x).Count();
            if (count >= 1)
                return true;

            return false;
        }


        /// <summary>
        /// Создание маленького сундука с предметом
        /// </summary>
        /// <param name="position"></param>
        /// <param name="id"></param>
        /// <param name="count"></param>
        void createcache(Vector3 position, int id, int count)
        {
            GameObject gameObject = Singleton<HNetworkManager>.Instance.NetInstantiate("WorldItemCache", position, Quaternion.identity, GameManager.GetSceneTime());
            Rigidbody component2 = gameObject.GetComponent<Rigidbody>();
            component2.position = gameObject.transform.position;
            component2.velocity = Vector3.zero;
            Inventory component3 = gameObject.GetComponent<Inventory>();
            component3.Capacity = 1;
            Singleton<GlobalItemManager>.Instance.GiveItem(Singleton<GlobalItemManager>.Instance.GetItem(id), count, component3);
        }


        /// <summary>
        /// Логи игроков, которые попали в рейдблок
        /// </summary>
        /// <param name="sess"></param>
        void Logs(List<PlayerSession> sess)
        {
            string msg = "[" + DateTime.Now + "]" + "Players are Raid Blocked:\n";
            foreach (var s in sess)
            {
                msg += s.Name + $" ({s.SteamId})" + "\n";
            }
            LogToFile("NoEscape", msg, this);
        }

        public bool IsValidSession(PlayerSession session)
        {
            return session?.SteamId != null && session.IsLoaded && session.Name != null && session.Identity != null &&
                   session.WorldPlayerEntity?.transform?.position != null;
        }

        #endregion
    }
}
